import logging
import time
import boto3

from cast_ai.se.models.cloud_confs import EksConfig
from cast_ai.se.models.execution_status import ExecutionStatus
from cast_ai.se.contollers.cloud_controller_svc import CloudController


class EKSController(CloudController):
    def __init__(self, aws_conf: EksConfig):
        try:
            self.conf = aws_conf
            self._logger = logging.getLogger(__name__)
            self._client = boto3.client(
                'autoscaling',
                region_name=self.conf.region,
                aws_access_key_id=self.conf.access_key,
                aws_secret_access_key=self.conf.access_secret_key
            )
        except Exception as e:
            self._logger.critical(f"An error occurred during EKSController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during EKSController initialization: {str(e)}")

    def disable_autoscaler(self) -> ExecutionStatus:
        self.scale(0)
        return ExecutionStatus()



    def get_node_count(self) -> int:
        ec2_client = boto3.client('ec2', region_name=self.conf.region,
                                  aws_access_key_id=self.conf.access_key,
                                  aws_secret_access_key=self.conf.access_secret_key)

        try:
            response = ec2_client.describe_instances(Filters=[
                {
                    'Name': 'tag:aws:autoscaling:groupName',
                    'Values': [f"{self.conf.asg}"]
                },
                {
                    'Name': f'tag:kubernetes.io/cluster/{self.conf.cluster_name}/nodegroup',
                    'Values': [f"{self.conf.ng}"]
                },
                {
                    'Name': 'instance-state-name',
                    'Values': ['running']
                }
            ])

            # Extract the count of running instances
            running_node_count = sum([len(reservations['Instances']) for reservations in response['Reservations']])
            return running_node_count
        except Exception as e:
            self._logger.exception(f"Error trying to get Nodes count: {str(e)}")
            raise RuntimeError(f"Error trying to get Nodes count: {str(e)}")

    def scale(self, node_count: int) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Scaling EKS Auto Scaling Group to {node_count} ]")

        try:
            for _ in range(5):
                response = self._client.update_auto_scaling_group(
                    AutoScalingGroupName=self.conf.asg,
                    MinSize=node_count,
                    MaxSize=node_count + 1,  # You can adjust the max size as needed
                    DesiredCapacity=node_count,
                )
                if response["ResponseMetadata"]["HTTPStatusCode"] == 200:
                    if not self._is_asg_scaled(node_count):
                        self._logger.warning(f"Default asg=({self.conf.asg}) has still not changed going to retry")
                        time.sleep(1)
                        continue
                    else:
                        self._logger.debug(f"Successfully updated EKS Auto Scaling Group"
                                           f" [{self.conf.asg}] : [{str(response)}]")
                        self._logger.info('Successfully updated EKS Auto Scaling Group')
                        return ExecutionStatus()
                else:
                    self._logger.error(f"Scaling default ng result = [{response}]")
                    return ExecutionStatus(f'HTTP {response["ResponseMetadata"]["HTTPStatusCode"]}')
        except Exception as e:
            self._logger.exception(f"Error trying to set Demo Nodes capacity: {str(e)}")
            raise RuntimeError(f"Error trying to set Demo Nodes capacity: {str(e)}")

    def _is_asg_scaled(self, node_count: int) -> bool:
        validate_asg_response = self._client.describe_auto_scaling_groups(AutoScalingGroupNames=[self.conf.asg])
        self._logger.debug(f"Validating ASG={validate_asg_response['AutoScalingGroups'][0]}")
        return (validate_asg_response["AutoScalingGroups"][0]["DesiredCapacity"] == node_count and
                validate_asg_response["AutoScalingGroups"][0]["MinSize"] == node_count)



import json
import logging
from typing import Dict, Any

from cast_ai.se.services.request_handle_svc import handle_request, CustomHTTPError, get_api_key_headers

logger = logging.getLogger(__name__)


# TODO: add logging
def cast_api_get(url: str, cast_api_key: str) -> Dict[str, Any]:
    headers = get_api_key_headers(cast_api_key)

    try:
        response = handle_request(url, headers, method="GET")
        return json.loads(response.content.decode('utf-8'))
    except CustomHTTPError as custom_err:
        raise custom_err


def cast_api_put(url: str, cast_api_key: str, data: Dict[str, Any]) -> Dict[str, Any]:
    headers = get_api_key_headers(cast_api_key)
    headers["Content-Type"] = "application/json"
    try:
        response = handle_request(url, headers, json=data, method="PUT")
        return json.loads(response.content.decode('utf-8'))
    except CustomHTTPError as custom_err:
        raise custom_err


def cast_api_post(url: str, cast_api_key: str, data: Dict[str, Any]) -> Dict[str, Any]:
    headers = get_api_key_headers(cast_api_key)
    try:
        response = handle_request(url, headers, json=data, method="POST")
        return json.loads(response.content.decode('utf-8'))
    except CustomHTTPError as custom_err:
        raise custom_err


def cast_api_delete(url: str, cast_api_key: str) -> Dict[str, Any]:
    headers = get_api_key_headers(cast_api_key)

    try:
        response = handle_request(url, headers, method="DELETE")
        return json.loads(response.content.decode('utf-8'))
    except CustomHTTPError as custom_err:
        logger.error(f"Error running Delete with url=[{url}] error=[{custom_err}]")
        # raise custom_err



import logging
from azure.mgmt.containerservice import ContainerServiceClient


from cast_ai.se.models.cloud_confs import AksConfig
from cast_ai.se.models.execution_status import ExecutionStatus
from cast_ai.se.contollers.cloud_controller_svc import CloudController


class AKSController(CloudController):
    def __init__(self, azure_conf: AksConfig):
        try:
            self._conf = azure_conf
            self._logger = logging.getLogger(__name__)
            self._client = ContainerServiceClient(self._conf.credential, self._conf.subscription_id)
            self._cluster = self._client.managed_clusters.get(self._conf.resource_group, self._conf.cluster_name)
        except Exception as e:
            self._logger.critical(f"An error occurred during AKSController initialization: {str(e)}")
            raise RuntimeError(f"An error occurred during AKSController initialization: {str(e)}")

    def disable_autoscaler(self) -> ExecutionStatus():
        try:
            node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
            node_pool.enable_auto_scaling = False
            node_pool.count = 1
            update_operation = self._client.managed_clusters.begin_create_or_update(self._conf.resource_group,
                                                                                    self._conf.cluster_name,
                                                                                    self._cluster)
            return ExecutionStatus()
        except Exception as e:
            self._logger.exception(f"Error trying to disable autoscaling on nodepool {self._conf.node_pool}: {str(e)}")
            raise RuntimeError(f"Error trying to disable autoscaling on nodepool {self._conf.node_pool}: {str(e)}")

    def get_node_count(self):
        try:
            node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
            return node_pool.count
        except Exception as e:
            self._logger.exception(f"Error trying to get Nodes count: {str(e)}")
            raise RuntimeError(f"Error trying to get Nodes count: {str(e)}")

    def scale(self, node_count: int) -> ExecutionStatus:

        self._logger.info(f"{'-' * 70}[ Scaling AKS Node Pool to {node_count} nodes ]")
        try:
            node_pool = next(pool for pool in self._cluster.agent_pool_profiles if pool.name == self._conf.node_pool)
            node_pool.count = node_count
            update_operation = self._client.managed_clusters.begin_create_or_update(self._conf.resource_group,
                                                                                    self._conf.cluster_name,
                                                                                    self._cluster)
            return ExecutionStatus()
        except Exception as e:
            self._logger.exception(f"Error trying to set Demo Nodes capacity: {str(e)}")
            raise RuntimeError(f"Error trying to set Demo Nodes capacity: {str(e)}")



import logging
from typing import Dict, Any

from cast_ai.se.constants import API_SERVER, EXTERNAL_CLUSTER_PREFIX, NODES_POSTFIX, CLUSTERS_PREFIX, POLICIES_POSTFIX, \
    AUDIT_PREFIX, CLUSTER_Q, RECONCILE_POSTFIX
from cast_ai.se.models.execution_status import ExecutionStatus
from cast_ai.se.services.api_requests_svc import cast_api_get, cast_api_put, cast_api_delete, cast_api_post


class CastController:
    def __init__(self, cast_api_key: str, default_cluster_id: str):
        self._logger = logging.getLogger(__name__)
        self._api_key = cast_api_key

        self._cluster_id = default_cluster_id
        self.cluster = None
        self._get_cluster_info()

    def _get_cluster_info(self):
        get_cluster_info_url = f"{API_SERVER}{EXTERNAL_CLUSTER_PREFIX}{self._cluster_id}"
        self.cluster = cast_api_get(get_cluster_info_url, self._api_key)

    def get_nodes(self):
        get_nodes_url = f"{API_SERVER}{EXTERNAL_CLUSTER_PREFIX}{self._cluster_id}{NODES_POSTFIX}"
        return cast_api_get(get_nodes_url, self._api_key)

    def delete_nodes(self, nodes) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Deleting Nodes via CAST API ]")
        # TODO: add failure logic (return)
        for node in nodes["items"]:
            delete_node_url = f"{API_SERVER}{EXTERNAL_CLUSTER_PREFIX}{self._cluster_id}{NODES_POSTFIX}/{node['id']}"
            cast_api_delete(delete_node_url, self._api_key)
        return ExecutionStatus()

    def enable_existing_policies(self) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Enabling CAST policies ]")
        current_policies = self.get_policies()
        if current_policies['enabled']:
            self._logger.warning("Current policies were already enabled")
            return ExecutionStatus()
        current_policies['enabled'] = True
        return self._set_policies(current_policies)

    def _set_policies(self, policies: Dict[str, Any]) -> ExecutionStatus:
        set_policies_url = f"{API_SERVER}{CLUSTERS_PREFIX}{self._cluster_id}{POLICIES_POSTFIX}"
        result = cast_api_put(set_policies_url, self._api_key, policies)
        # TODO: Add logic for failed execution
        return ExecutionStatus()

    def get_policies(self) -> Dict[str, Any]:
        get_policies_url = f"{API_SERVER}{CLUSTERS_PREFIX}{self._cluster_id}{POLICIES_POSTFIX}"
        return cast_api_get(get_policies_url, self._api_key)

    def are_policies_disabled(self) -> bool:
        try:
            return not self.get_policies()["enabled"]
        except Exception as e:
            raise Exception(e)

    def is_downscaler_disabled(self) -> bool:
        try:
            return not self.get_policies()["nodeDownscaler"]["enabled"]
        except Exception as e:
            raise Exception(e)

    def is_upscaler_disabled(self) -> bool:
        try:
            return not self.get_policies()["unschedulablePods"]["enabled"]
        except Exception as e:
            raise Exception(e)

    def disable_downscaler_policy(self) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Disabling Node deletion policy ]")
        current_policies = self.get_policies()
        if not current_policies['nodeDownscaler']["enabled"]:
            self._logger.warning("Node deletion policy already disabled")
            return ExecutionStatus()
        else:
            current_policies['nodeDownscaler']["enabled"] = False
            return self._set_policies(current_policies)

    def disable_unscheduled_pods_policy(self) -> ExecutionStatus:
        self._logger.info(f"{'-' * 70}[ Disabling Unscheduled pods policy ]")
        current_policies = self.get_policies()
        if not current_policies['unschedulablePods']["enabled"]:
            self._logger.warning("Unscheduled pod policy already disabled")
            return ExecutionStatus()
        else:
            current_policies['unschedulablePods']["enabled"] = False
            return self._set_policies(current_policies)

    def get_audit(self):
        get_cluster_info_url = f"{API_SERVER}{AUDIT_PREFIX}{CLUSTER_Q}{self._cluster_id}"
        return cast_api_get(get_cluster_info_url, self._api_key)

    def reconcile(self) -> ExecutionStatus:
        reconcile_url = f"{API_SERVER}{EXTERNAL_CLUSTER_PREFIX}{self._cluster_id}{RECONCILE_POSTFIX}"
        cast_api_post(reconcile_url, self._api_key, {})
        return ExecutionStatus()



import logging
from azure.identity import ClientSecretCredential


class EksConfig:
    def __init__(self, eks_conf: dict):
        self._logger = self._logger = logging.getLogger(__name__)
        try:
            self.region = eks_conf["REGION"]
            self.asg = eks_conf["AUTOSCALING_GROUP"]
            self.ng = eks_conf["NODE_GROUP"]
            self.cluster_name = eks_conf["CLUSTER_NAME"]
            self.access_key = eks_conf["ACCESS_KEY"]
            self.access_secret_key = eks_conf["ACCESS_SECRET_KEY"]
        except Exception as e:
            self._logger.critical(f"Was not able to initialize eks config:{str(e)}")
            raise RuntimeError(f"Was not able to initialize eks config:{str(e)}")


class AksConfig:
    def __init__(self, aks_conf: dict):
        self._logger = self._logger = logging.getLogger(__name__)
        try:
            self.tenant_id = aks_conf["TENANT_ID"]
            self.node_pool = aks_conf["NODE_POOL"]
            self.client_id = aks_conf["CLIENT_ID"]
            self.client_secret = aks_conf["CLIENT_SECRET"]
            self.credential = ClientSecretCredential(self.tenant_id, self.client_id, self.client_secret)
            self.subscription_id = aks_conf["SUBSCRIPTION_ID"]
            self.cluster_name = aks_conf["CLUSTER_NAME"]
            self.resource_group = aks_conf["RESOURCE_GROUP"]
        except Exception as e:
            self._logger.critical(f"Was not able to initialize aks config:{str(e)}")
            raise RuntimeError(f"Was not able to initialize aks config:{str(e)}")
